const Base = require('./Base');
const mysql = require('mysql2/promise');

module.exports = class DBHelper extends Base {
  constructor({ logName, dbConf }) {
    super({ logName: logName || '' });
    this.pool = mysql.createPool({
      ...dbConf,
    });
  }

  async query(requestId, ...args) {
    const startTime = Date.now();
    try {
      const data = await this.pool.query(...args);
      const endTime = Date.now();
      this.logger.debug(`${requestId}|query()|ARGS: ${args}|COST: ${endTime - startTime} MS`);
      return data;
    } catch (err) {
      const endTime = Date.now();
      this.logger.error(`${requestId}|query()|ARGS: ${args}|COST: ${endTime - startTime} MS|CODE: ${err.code}|STACK: ${err.stack}`);
      throw err;
    }
  }

  async transaction(requestId, func) {
    const conn = await this.pool.getConnection();
    const startTime = Date.now();
    try {
      this.logger.info(`${requestId}|transaction()|BEGIN`);
      await conn.beginTransaction();
      conn.queryWithLog = async (...args) => {
        try {
          const ret = await conn.query(...args);
          return ret;
        } catch (err) {
          this.logger.error(`${requestId}|ARGS: ${mysql.format(...args)}|MSG: ${err.message}|STACK: ${err.stack}`);
          throw err;
        }
      };
      const data = await func(conn);
      await conn.commit();
      this.logger.info(`${requestId}|transaction()|COMMIT`);
      return data;
    } catch (err) {
      this.logger.error(`${requestId}|transaction()|CODE: ${err.code}|STACK: ${err.stack}`);
      await conn.rollback();
      this.logger.info(`${requestId}|transaction()|ROLLBACK`);
      throw err;
    } finally {
      conn.release();
      this.logger.info(`${requestId}|transaction()|RELEASE|${Date.now() - startTime}`);
    }
  }
};
